@@ -49,6 +49,8 @@ void gl_event_view_show_detail (GlEventView *view);
 gboolean gl_event_view_handle_search_event (GlEventView *view, GAction *action, GdkEvent *event);
 void gl_event_view_set_search_mode (GlEventView *view, gboolean state);
 void gl_event_view_set_sort_order (GlEventView *view, GlSortOrder sort_order);
+GArray * gl_event_view_get_boot_ids (GlEventView *view);
+void gl_event_view_view_boot (GlEventView *view, const gchar *match);
 
 G_END_DECLS
 
