@@ -51,6 +51,7 @@ typedef struct
     GtkWidget *event_scrolled;
     GtkWidget *search_entry;
     gchar *search_text;
+    const gchar *boot_match;
 } GlEventViewListPrivate;
 
 G_DEFINE_TYPE_WITH_PRIVATE (GlEventViewList, gl_event_view_list, GTK_TYPE_BOX)
@@ -88,6 +89,9 @@ static gboolean
 search_in_result (GlJournalEntry *entry,
                   const gchar *search_text)
 {
+    gchar *field_name;
+    gchar *field_value;
+    gchar *search_text_copy;
     const gchar *comm;
     const gchar *message;
     const gchar *kernel_device;
@@ -98,14 +102,38 @@ search_in_result (GlJournalEntry *entry,
     kernel_device = gl_journal_entry_get_kernel_device (entry);
     audit_session = gl_journal_entry_get_audit_session (entry);
 
-    if ((comm ? strstr (comm, search_text) : NULL)
-        || (message ? strstr (message, search_text) : NULL)
-        || (kernel_device ? strstr (kernel_device, search_text) : NULL)
-        || (audit_session ? strstr (audit_session, search_text) : NULL))
+    search_text_copy = g_strdup (search_text);
+    field_name = strtok (search_text_copy, "=");
+    field_value = strtok (NULL, " ");
+
+    if (field_value == NULL)
     {
+        if ((comm ? strstr (comm, search_text) : NULL)
+            || (message ? strstr (message, search_text) : NULL)
+            || (kernel_device ? strstr (kernel_device, search_text) : NULL)
+            || (audit_session ? strstr (audit_session, search_text) : NULL))
+        {
+            g_free (search_text_copy);
+
+            return TRUE;
+        }
+    }
+    else if ((strstr ("_COMM", field_name) &&
+              comm ? strstr (comm, field_value) : NULL) ||
+             (strstr ("_MESSAGE", field_name) &&
+              message ? strstr (message, field_value) : NULL) ||
+             (strstr ("_KERNEL_DEVICE", field_name) &&
+              kernel_device ? strstr (kernel_device, field_value) : NULL) ||
+             (strstr ("_AUDIT_SESSION", field_name) &&
+              audit_session ? strstr (audit_session, field_value) : NULL))
+    {
+        g_free (search_text_copy);
+
         return TRUE;
     }
 
+    g_free (search_text_copy);
+
     return FALSE;
 }
 
@@ -173,27 +201,50 @@ listbox_search_filter_func (GtkListBoxRow *row,
         }
         else
         {
-            gchar *folded_search_term;
+            gchar *field_name;
+            gchar *field_value;
+            gchar *search_text_copy;
             const gchar *comm;
             const gchar *message;
             const gchar *kernel_device;
             const gchar *audit_session;
             gboolean matches;
 
-            folded_search_term = g_utf8_casefold (priv->search_text, -1);
-
             comm = gl_journal_entry_get_command_line (entry);
             message = gl_journal_entry_get_message (entry);
             kernel_device = gl_journal_entry_get_kernel_device (entry);
             audit_session = gl_journal_entry_get_audit_session (entry);
 
-            matches = (comm && utf8_strcasestr (comm, priv->search_text)) ||
-                      (message && utf8_strcasestr (message, priv->search_text)) ||
-                      (kernel_device && utf8_strcasestr (kernel_device, priv->search_text)) ||
-                      (audit_session && utf8_strcasestr (audit_session, priv->search_text));
+            search_text_copy = g_strdup (priv->search_text);
+            field_name = strtok (search_text_copy, "=");
+            field_value = strtok (NULL, " ");
 
-            g_free (folded_search_term);
-            return matches;
+            if (field_value == NULL)
+            {
+                matches = (comm && utf8_strcasestr (comm, priv->search_text)) ||
+                          (message && utf8_strcasestr (message, priv->search_text)) ||
+                          (kernel_device && utf8_strcasestr (kernel_device, priv->search_text)) ||
+                          (audit_session && utf8_strcasestr (audit_session, priv->search_text));
+
+                g_free (search_text_copy);
+
+                return matches;
+            }
+            else
+            {
+                matches = (utf8_strcasestr ("_comm", field_name) &&
+                           comm && strstr (comm, field_value)) ||
+                          (utf8_strcasestr ("_message", field_name) &&
+                           message && strstr (message, field_value)) ||
+                          (utf8_strcasestr ("_kernel_device", field_name) &&
+                           kernel_device && strstr (kernel_device, field_value)) ||
+                          (utf8_strcasestr ("_audit_session", field_name) &&
+                           audit_session && strstr (audit_session, field_value));
+
+                g_free (search_text_copy);
+
+                return matches;
+            }
         }
     }
 
@@ -243,6 +294,16 @@ gl_event_view_list_get_detail_entry (GlEventViewList *view)
     return priv->entry;
 }
 
+GArray *
+gl_event_view_list_get_boot_ids (GlEventViewList *view)
+{
+    GlEventViewListPrivate *priv;
+
+    priv = gl_event_view_list_get_instance_private (view);
+
+    return gl_journal_model_get_boot_ids (priv->journal_model);
+}
+
 gboolean
 gl_event_view_list_handle_search_event (GlEventViewList *view,
                                         GAction *action,
@@ -408,16 +469,18 @@ on_notify_category (GlCategoryList *list,
         case GL_CATEGORY_LIST_FILTER_IMPORTANT:
             {
               /* Alert or emergency priority. */
-              const gchar * query[] = { "PRIORITY=0", "PRIORITY=1", "PRIORITY=2", "PRIORITY=3", NULL };
+              const gchar * query[] = { "PRIORITY=0", "PRIORITY=1", "PRIORITY=2", "PRIORITY=3", NULL, NULL };
 
+              query[4] = priv->boot_match;
               gl_journal_model_set_matches (priv->journal_model, query);
             }
             break;
 
         case GL_CATEGORY_LIST_FILTER_ALL:
             {
-                const gchar *query[] = { NULL };
+                const gchar *query[] = { NULL, NULL };
 
+                query[0] = priv->boot_match;
                 gl_journal_model_set_matches (priv->journal_model, query);
             }
             break;
@@ -431,10 +494,12 @@ on_notify_category (GlCategoryList *list,
                                          "_TRANSPORT=stdout",
                                          "_TRANSPORT=syslog",
                                          NULL,
+                                         NULL,
                                          NULL };
 
                 uid_str = create_uid_match_string ();
                 query[3] = uid_str;
+                query[4] = priv->boot_match;
                 gl_journal_model_set_matches (priv->journal_model, query);
 
                 g_free (uid_str);
@@ -443,24 +508,27 @@ on_notify_category (GlCategoryList *list,
 
         case GL_CATEGORY_LIST_FILTER_SYSTEM:
             {
-                const gchar *query[] = { "_TRANSPORT=kernel", NULL };
+                const gchar *query[] = { "_TRANSPORT=kernel", NULL, NULL };
 
+                query[1] = priv->boot_match;
                 gl_journal_model_set_matches (priv->journal_model, query);
             }
             break;
 
         case GL_CATEGORY_LIST_FILTER_HARDWARE:
             {
-                const gchar *query[] = { "_TRANSPORT=kernel", "_KERNEL_DEVICE", NULL };
+                const gchar *query[] = { "_TRANSPORT=kernel", "_KERNEL_DEVICE", NULL, NULL };
 
+                query[2] = priv->boot_match;
                 gl_journal_model_set_matches (priv->journal_model, query);
             }
             break;
 
         case GL_CATEGORY_LIST_FILTER_SECURITY:
             {
-                const gchar *query[] = { "_AUDIT_SESSION", NULL };
+                const gchar *query[] = { "_AUDIT_SESSION", NULL, NULL };
 
+                query[1] = priv->boot_match;
                 gl_journal_model_set_matches (priv->journal_model, query);
             }
             break;
@@ -475,6 +543,21 @@ on_notify_category (GlCategoryList *list,
     gl_event_view_list_set_sort_order (view, sort_order);
 }
 
+void
+gl_event_view_list_view_boot (GlEventViewList *view, const gchar *match)
+{
+    GlEventViewListPrivate *priv;
+    GlCategoryList *categories;
+
+    g_return_if_fail (GL_EVENT_VIEW_LIST (view));
+
+    priv = gl_event_view_list_get_instance_private (view);
+    categories = GL_CATEGORY_LIST (priv->categories);
+    priv->boot_match = match;
+
+    on_notify_category (categories, NULL, view);
+}
+
 static gint
 gl_event_view_sort_by_ascending_time (GtkListBoxRow *row1,
                                       GtkListBoxRow *row2)
@@ -662,10 +745,13 @@ gl_event_view_list_init (GlEventViewList *view)
     gtk_widget_init_template (GTK_WIDGET (view));
 
     priv = gl_event_view_list_get_instance_private (view);
+
     priv->search_text = NULL;
-    priv->category_sizegroup = gtk_size_group_new (GTK_SIZE_GROUP_BOTH);
-    priv->message_sizegroup = gtk_size_group_new (GTK_SIZE_GROUP_BOTH);
-    priv->time_sizegroup = gtk_size_group_new (GTK_SIZE_GROUP_BOTH);
+    priv->boot_match = NULL;
+    priv->category_sizegroup = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
+    priv->message_sizegroup = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
+    priv->time_sizegroup = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
+
     categories = GL_CATEGORY_LIST (priv->categories);
 
     priv->journal_model = gl_journal_model_new ();
